@@ -1,50 +1,58 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 
 from tryton.gui.window.view_form.screen import Screen
-from interface import WidgetInterface
+from .widget import Widget
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
-from tryton.config import CONFIG
 import tryton.common as common
 import gettext
-from tryton.common import RPCExecute, RPCException
-from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.common.completion import get_completion, update_completion
+from tryton.common.domain_parser import quote
+from tryton.common.underline import set_underline
 
 _ = gettext.gettext
 
 
-class Many2Many(WidgetInterface):
+class Many2Many(Widget):
+    expand = True
 
-    def __init__(self, field_name, model_name, attrs=None):
-        super(Many2Many, self).__init__(field_name, model_name, attrs=attrs)
+    def __init__(self, view, attrs):
+        super(Many2Many, self).__init__(view, attrs)
 
-        self.widget = gtk.VBox(homogeneous=False, spacing=5)
+        self.widget = gtk.Frame()
+        self.widget.set_shadow_type(gtk.SHADOW_NONE)
+        self.widget.get_accessible().set_name(attrs.get('string', ''))
+        vbox = gtk.VBox(homogeneous=False, spacing=5)
+        self.widget.add(vbox)
         self._readonly = True
+        self._required = False
         self._position = 0
 
         hbox = gtk.HBox(homogeneous=False, spacing=0)
         hbox.set_border_width(2)
 
-        label = gtk.Label(attrs.get('string', ''))
-        label.set_alignment(0.0, 0.5)
-        hbox.pack_start(label, expand=True, fill=True)
+        self.title = gtk.Label(set_underline(attrs.get('string', '')))
+        self.title.set_use_underline(True)
+        self.title.set_alignment(0.0, 0.5)
+        hbox.pack_start(self.title, expand=True, fill=True)
 
         hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
         tooltips = common.Tooltips()
 
-        self.wid_text = PlaceholderEntry()
+        self.wid_text = gtk.Entry()
         self.wid_text.set_placeholder_text(_('Search'))
         self.wid_text.set_property('width_chars', 13)
-        self.wid_text.connect('focus-out-event', lambda *a: self._focus_out())
+        self.wid_text.connect('focus-out-event', self._focus_out)
         self.focus_out = True
         hbox.pack_start(self.wid_text, expand=True, fill=True)
 
         if int(self.attrs.get('completion', 1)):
-            self.wid_completion = get_completion()
+            self.wid_completion = get_completion(
+                create=self.attrs.get('create', True)
+                and common.MODELACCESS[self.attrs['relation']]['create'])
             self.wid_completion.connect('match-selected',
                 self._completion_match_selected)
             self.wid_completion.connect('action-activated',
@@ -57,22 +65,16 @@ class Many2Many(WidgetInterface):
         self.but_add = gtk.Button()
         tooltips.set_tip(self.but_add, _('Add existing record'))
         self.but_add.connect('clicked', self._sig_add)
-        img_add = gtk.Image()
-        img_add.set_from_stock('tryton-list-add',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_add.set_alignment(0.5, 0.5)
-        self.but_add.add(img_add)
+        self.but_add.add(common.IconFactory.get_image(
+                'tryton-add', gtk.ICON_SIZE_SMALL_TOOLBAR))
         self.but_add.set_relief(gtk.RELIEF_NONE)
         hbox.pack_start(self.but_add, expand=False, fill=False)
 
         self.but_remove = gtk.Button()
         tooltips.set_tip(self.but_remove, _('Remove selected record <Del>'))
         self.but_remove.connect('clicked', self._sig_remove)
-        img_remove = gtk.Image()
-        img_remove.set_from_stock('tryton-list-remove',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_remove.set_alignment(0.5, 0.5)
-        self.but_remove.add(img_remove)
+        self.but_remove.add(common.IconFactory.get_image(
+                'tryton-remove', gtk.ICON_SIZE_SMALL_TOOLBAR))
         self.but_remove.set_relief(gtk.RELIEF_NONE)
         hbox.pack_start(self.but_remove, expand=False, fill=False)
 
@@ -83,84 +85,67 @@ class Many2Many(WidgetInterface):
         frame = gtk.Frame()
         frame.add(hbox)
         frame.set_shadow_type(gtk.SHADOW_OUT)
-        self.widget.pack_start(frame, expand=False, fill=True)
+        vbox.pack_start(frame, expand=False, fill=True)
 
         self.screen = Screen(attrs['relation'],
             view_ids=attrs.get('view_ids', '').split(','),
             mode=['tree'], views_preload=attrs.get('views', {}),
-            row_activate=self._on_activate)
+            row_activate=self._on_activate,
+            limit=None)
         self.screen.signal_connect(self, 'record-message', self._sig_label)
 
-        self.widget.pack_start(self.screen.widget, expand=True, fill=True)
+        vbox.pack_start(self.screen.widget, expand=True, fill=True)
+
+        self.title.set_mnemonic_widget(
+            self.screen.current_view.mnemonic_widget)
 
         self.screen.widget.connect('key_press_event', self.on_keypress)
         self.wid_text.connect('key_press_event', self.on_keypress)
 
-    def _color_widget(self):
-        if hasattr(self.screen.current_view, 'widget_tree'):
-            return self.screen.current_view.widget_tree
-        return super(Many2Many, self)._color_widget()
-
-    def grab_focus(self):
-        return self.wid_text.grab_focus()
-
     def on_keypress(self, widget, event):
         editable = self.wid_text.get_editable()
         activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
+        remove_keys = [gtk.keysyms.Delete, gtk.keysyms.KP_Delete]
         if not self.wid_completion:
             activate_keys.append(gtk.keysyms.Return)
-        if event.keyval == gtk.keysyms.F3:
-            self._sig_add()
-            return True
-        if event.keyval == gtk.keysyms.F2 \
-                and widget == self.screen.widget:
-            self._sig_edit()
-            return True
-        if event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete) \
-                and widget == self.screen.widget:
-            self._sig_remove()
-            return True
-        if (widget == self.wid_text
-                and event.keyval in activate_keys
-                and editable
-                and self.wid_text.get_text()):
-            self._sig_add()
-            self.wid_text.grab_focus()
+        if widget == self.screen.widget:
+            if event.keyval == gtk.keysyms.F3 and editable:
+                self._sig_add()
+                return True
+            elif event.keyval == gtk.keysyms.F2:
+                self._sig_edit()
+                return True
+            elif event.keyval in remove_keys and editable:
+                self._sig_remove()
+                return True
+        elif widget == self.wid_text:
+            if event.keyval == gtk.keysyms.F3:
+                self._sig_new()
+                return True
+            elif event.keyval == gtk.keysyms.F2:
+                self._sig_add()
+                return True
+            elif event.keyval in activate_keys and self.wid_text.get_text():
+                self._sig_add()
+                self.wid_text.grab_focus()
         return False
 
     def destroy(self):
+        self.wid_text.disconnect_by_func(self._focus_out)
         self.screen.destroy()
 
-    def color_set(self, name):
-        super(Many2Many, self).color_set(name)
-        widget = self._color_widget()
-        # if the style to apply is different from readonly then insensitive
-        # cellrenderers should use the default insensitive color
-        if name != 'readonly':
-            widget.modify_text(gtk.STATE_INSENSITIVE,
-                    self.colors['text_color_insensitive'])
-
-    def _sig_add(self, *args, **kwargs):
+    def _sig_add(self, *args):
         if not self.focus_out:
             return
         domain = self.field.domain_get(self.record)
         add_remove = self.record.expr_eval(self.attrs.get('add_remove'))
         if add_remove:
             domain = [domain, add_remove]
-        context = self.field.context_get(self.record)
+        context = self.field.get_search_context(self.record)
+        order = self.field.get_search_order(self.record)
         value = self.wid_text.get_text()
 
         self.focus_out = False
-        try:
-            if value:
-                dom = [('rec_name', 'ilike', '%' + value + '%'), domain]
-            else:
-                dom = domain
-            ids = RPCExecute('model', self.attrs['relation'], 'search',
-                dom, 0, CONFIG['client.limit'], None, context=context)
-        except RPCException:
-            self.focus_out = True
-            return False
 
         def callback(result):
             self.focus_out = True
@@ -170,14 +155,17 @@ class Many2Many(WidgetInterface):
                 self.screen.display(res_id=ids[0])
             self.screen.set_cursor()
             self.wid_text.set_text('')
-        if len(ids) != 1 or not value or kwargs.get('win_search', False):
-            WinSearch(self.attrs['relation'], callback, sel_multi=True,
-                ids=ids, context=context, domain=domain,
-                view_ids=self.attrs.get('view_ids', '').split(','),
-                views_preload=self.attrs.get('views', {}),
-                new=self.attrs.get('create', True))
+        win = WinSearch(self.attrs['relation'], callback, sel_multi=True,
+            context=context, domain=domain, order=order,
+            view_ids=self.attrs.get('view_ids', '').split(','),
+            views_preload=self.attrs.get('views', {}),
+            new=self.attrs.get('create', True),
+            title=self.attrs.get('string'))
+        win.screen.search_filter(quote(value))
+        if len(win.screen.group) == 1:
+            win.response(None, gtk.RESPONSE_OK)
         else:
-            callback([(i, None) for i in ids])
+            win.show()
 
     def _sig_remove(self, *args):
         self.screen.remove(remove=True)
@@ -185,19 +173,25 @@ class Many2Many(WidgetInterface):
     def _on_activate(self):
         self._sig_edit()
 
+    def _get_screen_form(self):
+        domain = self.field.domain_get(self.record)
+        add_remove = self.record.expr_eval(self.attrs.get('add_remove'))
+        if add_remove:
+            domain = [domain, add_remove]
+        context = self.field.get_context(self.record)
+        # Remove the first tree view as mode is form only
+        view_ids = self.attrs.get('view_ids', '').split(',')[1:]
+        return Screen(self.attrs['relation'], domain=domain,
+            view_ids=view_ids,
+            mode=['form'], views_preload=self.attrs.get('views', {}),
+            context=context)
+
     def _sig_edit(self):
         if not self.screen.current_record:
             return
         # Create a new screen that is not linked to the parent otherwise on the
         # save of the record will trigger the save of the parent
-        domain = self.field.domain_get(self.record)
-        add_remove = self.record.expr_eval(self.attrs.get('add_remove'))
-        if add_remove:
-            domain = [domain, add_remove]
-        screen = Screen(self.attrs['relation'], domain=domain,
-            view_ids=self.attrs.get('view_ids', '').split(','),
-            mode=['form'], views_preload=self.attrs.get('views', {}),
-            readonly=self.attrs.get('readonly', False))
+        screen = self._get_screen_form()
         screen.load([self.screen.current_record.id])
 
         def callback(result):
@@ -205,11 +199,39 @@ class Many2Many(WidgetInterface):
                 screen.current_record.save()
                 # Force a reload on next display
                 self.screen.current_record.cancel()
-        WinForm(screen, callback)
+                # Force a display to clear the CellCache
+                self.screen.display()
+        WinForm(screen, callback, title=self.attrs.get('string'))
+
+    def _sig_new(self):
+        screen = self._get_screen_form()
+
+        def callback(result):
+            self.focus_out = True
+            if result:
+                record = screen.current_record
+                self.screen.load([record.id], modified=True)
+            self.wid_text.set_text('')
+            self.wid_text.grab_focus()
+
+        self.focus_out = False
+        WinForm(screen, callback, new=True, save_current=True,
+            title=self.attrs.get('string'), rec_name=self.wid_text.get_text())
 
     def _readonly_set(self, value):
         self._readonly = value
         self._set_button_sensitive()
+        self.wid_text.set_sensitive(not value)
+        self.wid_text.set_editable(not value)
+        self._set_label_state()
+
+    def _required_set(self, value):
+        self._required = value
+        self._set_label_state()
+
+    def _set_label_state(self):
+        common.apply_label_attributes(
+            self.title, self._readonly, self._required)
 
     def _set_button_sensitive(self):
         if self.record and self.field:
@@ -220,7 +242,6 @@ class Many2Many(WidgetInterface):
         else:
             size_limit = False
 
-        self.wid_text.set_sensitive(not self._readonly)
         self.but_add.set_sensitive(bool(
                 not self._readonly
                 and not size_limit))
@@ -271,25 +292,7 @@ class Many2Many(WidgetInterface):
 
     def _completion_action_activated(self, completion, index):
         if index == 0:
-            self._sig_add(win_search=True)
+            self._sig_add()
             self.wid_text.grab_focus()
         elif index == 1:
-            model = self.attrs['relation']
-            domain = self.field.domain_get(self.record)
-            add_remove = self.record.expr_eval(self.attrs.get('add_remove'))
-            if add_remove:
-                domain = [domain, add_remove]
-            context = self.field.context_get(self.record)
-
-            screen = Screen(model, domain, context=context, mode=['form'])
-
-            def callback(result):
-                self.focus_out = True
-                if result:
-                    record = screen.current_record
-                    self.screen.load([record.id], modified=True)
-                self.wid_text.set_text('')
-                self.wid_text.grab_focus()
-
-            self.focus_out = False
-            WinForm(screen, callback, new=True, save_current=True)
+            self._sig_new()
