@@ -1,8 +1,9 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-#This code is inspired by the pycha project
-#(http://www.lorenzogil.com/projects/pycha/)
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+# This code is inspired by the pycha project
+# (http://www.lorenzogil.com/projects/pycha/)
 import gtk
+
 from functools import reduce
 from tryton.common import hex2rgb, generateColorscheme, \
         COLOR_SCHEMES, datetime_strftime
@@ -15,7 +16,6 @@ import tryton.rpc as rpc
 import cairo
 from tryton.action import Action
 from tryton.gui.window import Window
-from tryton.translate import date_format
 
 
 class Popup(object):
@@ -35,7 +35,8 @@ class Popup(object):
 
     def set_position(self, widget, x, y):
         widget_x, widget_y = widget.window.get_origin()
-        width, height = widget.window.get_size()
+        allocation = widget.get_allocation()
+        width, height = allocation.width, allocation.height
         popup_width, popup_height = self.win.get_size()
         if x < popup_width // 2:
             x = popup_width // 2
@@ -69,10 +70,11 @@ class Popup(object):
 class Graph(gtk.DrawingArea):
     'Graph'
 
-    __gsignals__ = {"expose-event": "override"}
+    __gsignals__ = {"draw": "override"}
 
-    def __init__(self, xfield, yfields, attrs, model):
+    def __init__(self, view, xfield, yfields):
         super(Graph, self).__init__()
+        self.view = view
         self.xfield = xfield
         self.yfields = yfields
         self.datas = {}
@@ -84,8 +86,14 @@ class Graph(gtk.DrawingArea):
         self.connect('motion-notify-event', self.motion)
         self.connect('leave-notify-event', self.leave)
         self.popup = Popup(self)
-        self.attrs = attrs
-        self.model = model
+
+    @property
+    def attrs(self):
+        return self.view.attributes
+
+    @property
+    def model(self):
+        return self.view.screen.model_name
 
     def destroy(self):
         self.popup.destroy()
@@ -97,23 +105,17 @@ class Graph(gtk.DrawingArea):
     def leave(self, widget, event):
         self.popup.hide()
 
-    # Handle the expose-event by drawing
-    def do_expose_event(self, event):
-
-        # Create the cairo context
+    def do_draw(self, cr):
         cr = self.window.cairo_create()
+        width = self.get_allocated_width()
+        height = self.get_allocated_height()
 
-        # Restrict Cairo to the exposed area; avoid extra work
-        cr.rectangle(event.area.x, event.area.y,
-                event.area.width, event.area.height)
-        cr.clip()
-
-        self.updateArea(cr, *self.window.get_size())
-        self.drawBackground(cr, *self.window.get_size())
-        self.drawLines(cr, *self.window.get_size())
-        self.drawGraph(cr, *self.window.get_size())
-        self.drawAxis(cr, *self.window.get_size())
-        self.drawLegend(cr, *self.window.get_size())
+        self.updateArea(cr, width, height)
+        self.drawBackground(cr, width, height)
+        self.drawLines(cr, width, height)
+        self.drawGraph(cr, width, height)
+        self.drawAxis(cr, width, height)
+        self.drawLegend(cr, width, height)
 
     def export_png(self, filename, width, height):
         surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
@@ -125,7 +127,7 @@ class Graph(gtk.DrawingArea):
         self.drawGraph(cx, width, height)
         self.drawAxis(cx, width, height)
         self.drawLegend(cx, width, height)
-        surface.write_to_png(filename.encode('utf-8'))
+        surface.write_to_png(filename)
 
         self.queue_draw()
 
@@ -135,7 +137,7 @@ class Graph(gtk.DrawingArea):
     def action_keyword(self, ids):
         if not ids:
             return
-        ctx = self.group.context.copy()
+        ctx = self.group._context.copy()
         if 'active_ids' in ctx:
             del ctx['active_ids']
         if 'active_id' in ctx:
@@ -174,17 +176,17 @@ class Graph(gtk.DrawingArea):
             base = 1
         else:
             base = 10 ** int(math.log(self.yrange, 10))
-        for i in xrange(int(self.yrange / base) + 1):
+        for i in range(int(self.yrange / base) + 1):
             val = int(self.minyval / base) * base + i * base
             h = (val - self.minyval) * self.yscale
-            label = locale.format('%.2f', val, True)
+            label = locale.localize('{:.2f}'.format(val), True)
             ylabels.append((h, label))
         return ylabels
 
     def XLabels(self):
         xlabels = []
         i = 0.0
-        keys = self.datas.keys()
+        keys = list(self.datas.keys())
         keys.sort()
         for key in keys:
             if self.xrange == 0:
@@ -197,7 +199,7 @@ class Graph(gtk.DrawingArea):
 
     def drawAxis(self, cr, width, height):
         cr.set_source_rgb(*hex2rgb('#000000'))
-        cr.set_line_width(1.5)
+        cr.set_line_width(0.5)
 
         # Y axis
         def drawYLabel(h, label):
@@ -266,9 +268,10 @@ class Graph(gtk.DrawingArea):
             x2 = x1 + self.area.w
 
         cr.save()
-        cr.set_source_rgb(*hex2rgb('#808080'))
-        cr.set_line_width(1.5)
+        cr.set_source_rgb(*hex2rgb('#A0A0A0'))
+        cr.set_line_width(0.3)
         cr.new_path()
+        cr.set_dash([5.0, 4.0])
         cr.move_to(x1, y1)
         cr.line_to(x2, y2)
         cr.close_path()
@@ -301,7 +304,7 @@ class Graph(gtk.DrawingArea):
         cr.rectangle(pos_x, pos_y, width, height)
         cr.set_source_rgba(1, 1, 1, 0.8)
         cr.fill_preserve()
-        cr.set_line_width(2)
+        cr.set_line_width(0.5)
         cr.set_source_rgb(*hex2rgb('#000000'))
         cr.stroke()
 
@@ -315,7 +318,7 @@ class Graph(gtk.DrawingArea):
                     y + bullet / 2.0 + text_height / 2.0)
             cr.show_text(keys2txt[key])
 
-        cr.set_line_width(1)
+        cr.set_line_width(0.5)
         x = pos_x + padding
         y = pos_y + padding
         for key in keys:
@@ -379,12 +382,16 @@ class Graph(gtk.DrawingArea):
                 if yfield['name'] == '#':
                     self.datas[x][key] += 1
                 else:
-                    self.datas[x][key] += \
-                        float(model[yfield['name']].get(model) or 0)
+                    value = model[yfield['name']].get(model)
+                    if isinstance(value, datetime.timedelta):
+                        value = value.total_seconds()
+                    self.datas[x][key] += float(value or 0)
+        date_format = self.view.screen.context.get('date_format', '%x')
+        datetime_format = date_format + ' %X'
         if isinstance(minx, datetime.datetime):
             date = minx
             while date <= maxx:
-                self.labels[date] = datetime_strftime(date, date_format())
+                self.labels[date] = datetime_strftime(date, datetime_format)
                 self.datas.setdefault(date, {})
                 for yfield in self.yfields:
                     self.datas[date].setdefault(
@@ -393,7 +400,7 @@ class Graph(gtk.DrawingArea):
         elif isinstance(minx, datetime.date):
             date = minx
             while date <= maxx:
-                self.labels[date] = datetime_strftime(date, date_format())
+                self.labels[date] = datetime_strftime(date, date_format)
                 self.datas.setdefault(date, {})
                 for yfield in self.yfields:
                     self.datas[date].setdefault(
@@ -432,7 +439,7 @@ class Graph(gtk.DrawingArea):
         else:
             self.xscale = 1.0 / self.xrange
 
-        if not self.datas.values():
+        if not list(self.datas.values()):
             self.maxyval = 0.0
             self.minyval = 0.0
         else:
@@ -457,7 +464,7 @@ class Graph(gtk.DrawingArea):
         color = self.attrs.get('color', 'blue')
         r, g, b = hex2rgb(COLOR_SCHEMES.get(color, color))
         maxcolor = max(max(r, g), b)
-        self.colorScheme = generateColorscheme(color, keys + ['__highlight'],
+        self.colorScheme = generateColorscheme(color, keys,
                 maxcolor / (len(keys) or 1))
         for yfield in self.yfields:
             if yfield.get('color'):
